<?php
#CMS - CMS Made Simple
#(c)2008-2010 by Nicholas Vrtis (vrtisworks@users.sf.net)
#This project's homepage is: http://www.vrtisworks.com/cmsms/
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// This is an adaptation of ideas from a number of poetry writing programs I
// found.  Sorry to not give credit, but don't remember where I found them.
// I wanted a program to generate a phrase that I could use the first letter
// of each word as part of a password.  Some people can remember phrases easier
// than a random string of characters.
function smarty_cms_function_passphrasemaker($params, &$smarty) {

    if (isset($_POST['len'])) {
        $len=sprintf('%d',$_POST['len']);
        if ($len<3) {
            $len=3;
        }
    } else {
        $len=8;
    }
    if (isset($_POST['nbrspcl'])) {
        $nbrspcl=sprintf('%d',$_POST['nbrspcl']);
        if ($nbrspcl<0) {
            $nbrspcl=3;
        }
        if ($nbrspcl>$len) {
            $nbrspcl=$len;
        }
    } else {
        $nbrspcl=3;
    }
?>
  <form action="<?php $_SERVER['PHP_SELF'].'?'.$_SERVER['QUERY_STRING'] ?>" method="post" name="passwordmakerForm">
<div class="formbox">
    <div class="formrow">
      <span class="formlabel">Length:</span>
      <input type="text" class="forminput" size="3" name="len" value="<?php echo $len;?>" />
    </div>
    <div class="formrow">
      <span class="formlabel">Number special chars:</span>
     <input type="text" class="forminput" size="3" name="nbrspcl" value="<?php echo $nbrspcl;?>" />
    </div>
    <div class="formrow">
        <input type="submit" class="formsubmit" name="doit" value="Generate" />
    </div>
</div>
 </form>
<br />
<table class="wordsbox">
<?php
    //require_once '../passwordmaker/WordBox.php';
    $subs= array('a'=>'*','b'=>'6','c'=>'(','d'=>'-','e'=>'3','f'=>'4','g'=>'9','h'=>'H','i'=>'1','j'=>'/','k'=>'<','l'=>'7',
        'm'=>'>','n'=>'#','o'=>'0','p'=>'+','q'=>'?','r'=>'^','s'=>'$','t'=>'2','u'=>'_','v'=>'5','w'=>':','x'=>'!','y'=>'?','z'=>'@');
    $pp=new PasswordPhrase();
    for ($i=0;$i<8; $i++) {
        $pp->PPgenerate($len);
        $cnt=count($pp->wordList);
        echo '<tr><td>';
        for ($j=0; $j<$cnt; $j++) {
            echo htmlentities($pp->wordList[$j], ENT_QUOTES),'&nbsp;';
        }
        echo '</td></tr>';
        $baseword=implode('',$pp->charList);
        $password=$baseword;
        if ($nbrspcl>0) {
            $n=$nbrspcl;
            while ($n>0) {
                $idx=rand(1,$len)-1;
                if ($baseword[$idx]==$password[$idx]) {
                    //If they are still == then we haven't done a substitution yet.
                    //Flip a coin heads=Make it a capital, tails=substitute
                    if (rand(0,1)==0) {
                        //Substitue a special character
                        $password[$idx]=$subs[$password[$idx]];
                    } else {
                        //Make it a capital
                        $password[$idx]=strtoupper($password[$idx]);
                    }
                    $n--;
                }
            }
        }
        if ($nbrspcl==0) {
            //No special processing.. just do the one version...
            echo '<tr><td>',htmlentities($baseword, ENT_QUOTES),'</td></tr>',"\n";
        } else {
            echo '<tr><td>',htmlentities($baseword, ENT_QUOTES),'&nbsp;&nbsp;=&gt;&nbsp;&nbsp;',htmlentities($password, ENT_QUOTES),'</td></tr>',"\n";
        }
    }
    echo '</table>';
}    

function smarty_cms_help_function_passphrasemaker() {
  ?>
  <h3>What does this do?</h3>
  <p>Generates a phrase that can be used as a password, with and without special character substitutions.</p>
  <h3>How do I use it?</h3>
  <p>Just insert the tag into your template/page like: <code>{passphrasemaker}</code><br>
  <p>The following CSS classes are used:
  <ul>
      <li><em>formbox</em> The outer DIV that marks the parameter form (where they can
         change the defaults for the list of passwords that gets generated.</li>
        <li><em>formrow</em> Each row in the parameter form.</li>
        <li><em>formlabel</em> The label part of each parameter.</li>
        <li><em>forminput</em> The input part of each parameter.</li>
        <li><em>formsubmit</em> The Submit button.</li>
        <li><em>wordsbox</em> The table where the resulting generated words are put</li>
  </ul>
  </p>
  <h3>What parameters does it take?</h3>
  <ul>
    <li>None</li>
  </ul>
  </p>
  <?php
}

function smarty_cms_about_function_passphrasemaker() {
  ?>
  <p>Author: Nicholas Vrtis&lt;vrtisworks@users.sourceforge.net&gt;</p>
  <p>Version: 1.0</p>
  <p>
  Change History:<br/>
  None
  </p>
  <?php
}

class PasswordPhrase {

//The words don't need to be in alpha order, but it helps avoid duplicates
//NOTE: These need to be such that 'singular' just adds an 's' on the end
    var $verbs = array("argue","bite","buy","build","crash","cut","drink",
    "drive","drop","eat","enter","exhale","fall","find","follow","get",
    "give","grasp","hit","hurt","ignore","imagine","jump","jab","kick","kneel",
    "lick","lift","look","melt","mold","nag","order","open","overflow",
    "place","plead","quiz","read","run","search","sing","sip","slam","smell","take",
    "touch","understand","usurp","vex","wave","whisper");
    var $verbsUsed;
    var $articles = array("a","all","each","her","his","many","my","no",
    "one","some","that","the","their","this","your");
    var $articlesUsed;
    var $adjectives = array("ancient","angry","blue","bland","clean",
    "cold","cool","dirty","dark","effective","evil","filthy","flat","folded","forbidden",
    "green","greasy","hot","icy","juicy","kind","likable","lovely",
    "moldy","necessary","nice","old","odd","plastic","pretty","quiet","quick",
    "red","round","slippery","smelly","tame","timid","uniform","upright",
    "vibrant","wild","white","yellow");
    var $adjectivesUsed;
    //NOTE: These need to be such that plural is just an 's' on the end
    var $nouns = array("arm","apple","bear","boat","boy",
    "book","bottle","car","cart","cat","chicken","clock","computer","cow","dog","door",
    "eagle","elbow","finger","file","ghost","girl","hose","hotel","image","jack","jar",
    "key","kite","lamp","map","mixer","mother","name","napkin","network","nose",
    "oar","opener","pear","printer","quill","rat","rainbow","sheet","shoe",
    "snake","soap","train","truck","thumb","umbrella","uncle","vessel",
    "video","vowel","wrench","window","xray","zebra");
    var $nounsUsed;
    var $adverbs = array("abruptly","always","basically","brightly",
    "cheerfully","deliberately","eagerly","finally","grimly","happily",
    "hysterically","immediately","immensely","justly","kindly",
    "lightly","loudly","moodily","never","painfully","quietly","quickly",
    "rapidly","rarely","remorselessly","sadly","slowly","suddenly","timidly",
    "uncomfortably","undesirably","wetly","wildly");
    var $adverbsUsed;

    var $VOWELS="aeiou";
    var $resultSize;
    var $wordList;
    var $charList;
    var $wordPosition;

    function PPgenerate($sz) {

        $this->resultSize = $sz;			//Save the size of the resulting word
        $this->verbsUsed = array();			//Clear out the 'used' arrays.
        $this->articlesUsed = array();
        $this->adjectivesUsed = array();
        $this->nounsUsed = array();
        $this->adverbsUsed = array();
        $this->wordList = array();
        $this->charList = array();
        if (rand(0,10)>5) {					//Random choice of which type of phrase
            return $this->newFormat1Word();
        } else {
            return $this->newFormat2Word();
        }
    }

    function newFormat1Word() {
        if ($this->resultSize >12 ||
            $this->resultSize <5) {
            return $this->newFormat2Word();
        }
        $opts = array(3,3,1);				//0=adj1; 1=adj2; 2=adv
        //Format 1 is 'Art' 'Adj(0-3)' 'Noun' 'Adv(0-1)' 'Verb' 'Art' 'Adj(0-3)' 'Noun' '.'
        $length = 12;				//This is starting length
        while ($length>$this->resultSize) {	//Trim total to requested Size
            $x = rand(0,2);			//Pick one to trim
            if ($opts[$x]>0) {		//Make sure got some to remove
                $opts[$x]--;		//Adjust the counters
                $length--;
            }
        }


        $this->wordPosition=0;
        //Start with article
        $articlePosition = $this->wordPosition;
        $aWord = $this->pickAWord($this->articles,$this->articlesUsed);
        $plural=$this->isPlural($aWord);
        $this->addAWord($aWord);
        //Then maybe some adjectives
        while ($opts[0]>0) {
            $this->addAWord($this->pickAWord($this->adjectives,$this->adjectivesUsed));
            $opts[0]--;
        }
        $aWord = $this->pickAWord($this->nouns,$this->nounsUsed);
        if ($plural)
            $this->addAWord($aWord.'s');
        else
            $this->addAWord($aWord);
        //Adjust if 'A' 'vowel'
        $a = substr($this->wordList[$articlePosition+1],0,1);	//Get 1st char of next word
        if ($this->wordList[$articlePosition]=='A' &&
            strpos($this->VOWELS,$a)!==false)
            $this->wordList[$articlePosition]='An';
        //Then maybe some adverbs
        while ($opts[2]>0) {
            $this->addAWord($this->pickAWord($this->adverbs,$this->adverbsUsed));
            $opts[2]--;
        }
        //Then the verb (might need to adjust for plural noun subject
        if ($plural)
            $this->addAWord($this->pickAWord($this->verbs,$this->verbsUsed));
        else
            $this->addAWord($this->pickAWord($this->verbs,$this->verbsUsed).'s');
        //Then another article
        $articlePosition = $this->wordPosition;
        $aWord = $this->pickAWord($this->articles,$this->articlesUsed);
        $plural=$this->isPlural($aWord);
        $this->addAWord($aWord);
        //Then maybe some adjectives
        while ($opts[1]>0) {
            $this->addAWord($this->pickAWord($this->adjectives,$this->adjectivesUsed));
            $opts[1]--;
        }
        $aWord = $this->pickAWord($this->nouns,$this->nounsUsed);
        if ($plural)
            $this->addAWord($aWord.'s');
        else
            $this->addAWord($aWord);
        //Adjust if 'a' 'vowel'
        $a=substr($this->wordList[$articlePostion+1],0,1);
        if ($this->wordList[$articlePosition]=='a' &&
            strpos($this->VOWELS,$a)!==false)
            $this->wordList[$articlePosition]='an';
        $this->wordList[$this->wordPosition-1].='.';
    }

    function newFormat2Word() {
        if ($this->resultSize<3) {
            return;
        }
        if ($this->resultSize>7) {				//Max size for this format is 7
            return $this->newFormat1Word();			//Use format 1 if min is too big
        }

        //Format 2 is 'Art' 'Adj(0-3)' 'Noun' 'Adv(0-1)' 'Verb' '!'
        $opts=array(3,1);			//0=adj1; 2=adv
        $length =7;					//See what total length is
        while ($length>$this->resultSize) {	//Trim total to requested size
            $x = rand(0,1);			//Pick one to trim
            if ($opts[$x]>0) {
                $opts[$x]--;
                $length--;
            }
        }

        $this->wordPosition=0;
        //An article
        $articlePosition = $this->wordPosition;
        $aWord = $this->pickAWord($this->articles,$this->articlesUsed);
        $plural=$this->isPlural($aWord);
        $this->addAWord($aWord);
        //Then maybe some adjectives
        while ($opts[0]>0) {
            $this->addAWord($this->pickAWord($this->adjectives,$this->adjectivesUsed));
            $opts[0]--;
        }
        //Then a noun (might be plural or a digit)
        $aWord = $this->pickAWord($this->nouns,$this->nounsUsed);
        if ($plural)
            $this->addAWord($aWord.'s');
        else
            $this->addAWord($aWord);
        //Adjust if 'A' 'vowel'
        $a = substr($this->wordList[$articlePosition+1],0,1);	//Get 1st char of next word
        if ($this->wordList[$articlePosition]=='a' &&
            strpos($this->VOWELS,$a)!==false)
            $this->wordList[$articlePosition]='an';
        //Maybe some adverbs
        while ($opts[1]>0) {
            $this->addAWord($this->pickAWord($this->adverbs,$this->adverbsUsed));
            $opts[1]--;
        }
        //Then the verb
        if($plural)
            $this->addAWord($this->pickAWord($this->verbs,$this->verbsUsed));
        else
            $this->addAWord($this->pickAWord($this->verbs,$this->verbsUsed).'s');
        $this->wordList[$this->wordPosition-1].='!';
    }

    function addAWord($word) {

        $this->charList[$this->wordPosition]=substr($word,0,1);
        if ($this->wordPosition==0)
            $this->wordList[$this->wordPosition]= ucfirst($word);
        else
            $this->wordList[$this->wordPosition]=$word;
        $this->wordPosition++;
    }

    function isPlural($word) {
        if ($word=='all' ||
            $word=='their' ||
            $word=='many' ||
            $word=='some')
            return true;
        else
            return false;
    }

    function pickAWord($words, & $used) {
        $x=count($words)-1;
        $z=rand(0,$x);		//Pick the nth unused word
        //Not necessarily the most efficient, but we won't be using a high percentage
        while ($used[$z]) {
            $z+=3;
            if ($z>$x) $z-=$x;
        }
        $used[$z]=true;		//So we know we used it next time..
        return $words[$z];
    }
}

